SystemOrganization addCategory: #'Scripting-Examples'!
SystemOrganization addCategory: #Scripting!

Object subclass: #HeadlessSave
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Scripting'!

!HeadlessSave class methodsFor: 'script entry' stamp: 'StefanMarr 4/5/2012 21:47'!
run: arguments 
	| proc |
	proc := [
		| basicName newName |
		ScriptConsole println: 'Do wait, let system run non-busy, and then save and quit'.
		(Delay forSeconds: 10) wait.
		ScriptConsole println: 'Do headless save now.'.
		basicName := SmalltalkImage current imageName withoutLast: '.image' size.
		newName := basicName, '-headless.image'.
		SmalltalkImage current saveAs: newName.
		SmalltalkImage current snapshot: false andQuit: true.
		] newProcess.
	
	proc priority: Processor userBackgroundPriority.
	proc resume.
! !

Object subclass: #HelloWorld
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Scripting-Examples'!

!HelloWorld commentStamp: 'StefanMarr 5/14/2011 16:42' prior: 0!
Simple example for how to use ScriptStarter.

HelloWorld prints all arguments given to #run:.!

!HelloWorld class methodsFor: 'script entry' stamp: ''!
run: arguments 
	ScriptConsole print: 'Hello World!!'.
	arguments
		do: [:elem | ScriptConsole print: elem].
	SmalltalkImage current snapshot: false andQuit: true! !

Object subclass: #Run
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Scripting'!

!Run commentStamp: 'StefanMarr 3/19/2012 21:24' prior: 0!
Run is a class that will take the command line arguments and will try to evaluate it as Smalltalk.!

!Run class methodsFor: 'script entry' stamp: 'StefanMarr 3/19/2012 21:47'!
run: args
	| str cmdClass |
	ScriptStarter quitVMAfterScriptExecution: false.
	
	cmdClass := args removeFirst.
	str := args reduce: [:a :b | a asString, ' ', b asString ].
	ScriptConsole println: str.
	ScriptConsole println: args asString.
	ScriptConsole println: 'Going to evaluate the string'.
	[ Compiler evaluate: str. ]
		ifError: [:err :rcvr | ScriptConsole println: 'Error: ', err].! !

Object subclass: #ScriptConsole
	instanceVariableNames: ''
	classVariableNames: 'IsRVM LastPrintedChar OutputToTranscript Position PrintTarget'
	poolDictionaries: ''
	category: 'Scripting'!

!ScriptConsole class methodsFor: 'streaming' stamp: ''!
<< aString
	self print: aString! !

!ScriptConsole class methodsFor: 'printing' stamp: ''!
cr
	self println: ''.! !

!ScriptConsole class methodsFor: 'initialization' stamp: 'StefanMarr 4/5/2012 22:08'!
detectAvailableMechanismForStdout
	"The prefered means is to use the stdout file handle.
	 Will try to fall back on the RoarVM primitive if necessary."
	| cls |
	(FileStream respondsTo: #stdout)
		ifTrue: [
			PrintTarget := FileStream stdout.
			IsRVM := false. "Not needed when stdout is used"
			^ self.
		].
		
	cls := Smalltalk at: #RVMOperations ifAbsent: [nil].
	IsRVM := (cls notNil and: [cls perform: #isRVM]).
	IsRVM 
		ifTrue: [ 
			PrintTarget := cls.
			^ self.
		]
		ifFalse: [
			"Squeak 4.2 provides a stream at FileStream>>stdout"
			
		].
	
	PrintTarget := Transcript.
	^ Transcript! !

!ScriptConsole class methodsFor: 'initialization' stamp: 'StefanMarr 12/31/2011 01:27'!
initialize
	"ScriptConsole initialize"
	OutputToTranscript := false.
	self detectAvailableMechanismForStdout.
	Position := 0.! !

!ScriptConsole class methodsFor: 'streaming' stamp: 'StefanMarr 12/5/2011 18:11'!
nextPut: somethingAsString
	self print: somethingAsString! !

!ScriptConsole class methodsFor: 'streaming' stamp: 'StefanMarr 12/31/2011 01:23'!
nextPutAll: somethingAsString
	self print: somethingAsString! !

!ScriptConsole class methodsFor: 'printing' stamp: ''!
outputToTranscript
	^ OutputToTranscript! !

!ScriptConsole class methodsFor: 'printing' stamp: ''!
outputToTranscript: aBool
	OutputToTranscript := aBool! !

!ScriptConsole class methodsFor: 'read, write, position' stamp: 'StefanMarr 12/31/2011 01:29'!
peekLast
	^ LastPrintedChar! !

!ScriptConsole class methodsFor: 'read, write, position' stamp: 'StefanMarr 12/31/2011 01:27'!
position
	^ Position! !

!ScriptConsole class methodsFor: 'printing' stamp: ''!
print: somethingAsString
	self print: somethingAsString withLineEnding: ''.! !

!ScriptConsole class methodsFor: 'printing' stamp: 'StefanMarr 12/31/2011 01:30'!
print: somethingAsString withLineEnding: lineEnd
	"Will output the somethingAsString to stdout using one of the available mechansims and to the transcript if requested"
	| output |
	output := somethingAsString asString.
	Position := Position + output size.
	output size > 0 ifTrue: [
		LastPrintedChar := output last].
	
	PrintTarget ifNotNil: [
		IsRVM ifTrue: [ PrintTarget print: output. ]
			   ifFalse: [ PrintTarget nextPutAll: (output, lineEnd) ].
	].

	(OutputToTranscript or: [PrintTarget isNil])
		ifTrue: [
			Transcript show: output, lineEnd.
		].! !

!ScriptConsole class methodsFor: 'printing' stamp: ''!
println: somethingAsString
	self print: somethingAsString withLineEnding: String crlf.! !

!ScriptConsole class methodsFor: 'printing' stamp: 'StefanMarr 12/31/2011 01:22'!
space
	"Append a space character to the receiver."

	self nextPut: Character space! !

!ScriptConsole class methodsFor: 'printing' stamp: 'StefanMarr 12/31/2011 01:27'!
tab
	"Append a tab character to the receiver."

	self nextPut: Character tab! !

Object subclass: #ScriptStarter
	instanceVariableNames: ''
	classVariableNames: 'IsResuming QuitVM SafeBeforeQuit'
	poolDictionaries: ''
	category: 'Scripting'!

!ScriptStarter commentStamp: 'StefanMarr 5/14/2011 16:29' prior: 0!
ScriptStarter provides a startUp method and registers itself on the startUpList to enable scripting using command-line arguments.

The first argument on the command-line is used to identify a class on which #run: is invoked with the arguments as an array.

#run: can be used similar to classic main(String[] args) methods in other languages. !

!ScriptStarter class methodsFor: 'helper' stamp: 'StefanMarr 3/19/2012 21:44'!
executeRunFor: scriptClassName with: arguments
	| scriptClass |
	scriptClass := Smalltalk classNamed: scriptClassName.
	scriptClass ifNotNil: [
		scriptClass run: arguments.
		QuitVM ifTrue: [
			Smalltalk snapshot: SafeBeforeQuit andQuit: true.
		].
	].! !

!ScriptStarter class methodsFor: 'helper' stamp: ''!
getSystemAttributes
	| arguments arg i |
	arguments := OrderedCollection new.
	i := 2.
	[[arg := SmalltalkImage current getSystemAttribute: i] value == nil]
		whileFalse: [arguments addLast: arg.
			i := i + 1].
	^ arguments! !

!ScriptStarter class methodsFor: 'initialization' stamp: 'sm 5/30/2011 22:33'!
initialize
	"Automatically install ScriptStarter when the code is filed in."
	self install.
	
	QuitVM := true.
	SafeBeforeQuit := false.
	IsResuming := false.! !

!ScriptStarter class methodsFor: 'system startup entry' stamp: 'sm 5/25/2011 12:41'!
install
	"Will register ScriptStarter on the StartUpList to activate it on VM startup."
	| pref |
	Smalltalk addToStartUpList: ScriptStarter.
	
	"Disable the loading of documents on startup"
	pref := Smalltalk at: #Preferences ifAbsent: [nil].
	pref	ifNotNil: [
			(pref respondsTo: #disable:)
				ifTrue: [	pref disable: #readDocumentAtStartup. ]
			]
		ifNil: [
			(Smalltalk globals at: #AbstractLauncer ifAbsent: [nil])
				ifNotNilDo: [:launcher |
					launcher readDocumentAtStartup: false
				]
			].! !

!ScriptStarter class methodsFor: 'configuration' stamp: ''!
quitVMAfterScriptExecution
	^ QuitVM! !

!ScriptStarter class methodsFor: 'configuration' stamp: ''!
quitVMAfterScriptExecution: aBool
	QuitVM := aBool! !

!ScriptStarter class methodsFor: 'configuration' stamp: ''!
saveImageBeforeQuit
	^ SafeBeforeQuit! !

!ScriptStarter class methodsFor: 'configuration' stamp: ''!
saveImageBeforeQuit: aBool
	SafeBeforeQuit := aBool! !

!ScriptStarter class methodsFor: 'system startup entry' stamp: 'sm 5/30/2011 22:35'!
startUp
	"startUp is invoked during startup.
	 ScriptStarter has to be registered in the SystemDirectory's StartUpList
	 See ScriptStarter>>install"
	
	| arguments scriptClassName |
	"Ok, only, and really only during startup"
	IsResuming ifFalse: [ Transcript show: 'ScriptStarter did not execute a given script, since it assumes to be executed after a snapshot and not during startup.'.
		^ self. ].
	
	"First make sure that stdout output can be used if available."
	ScriptConsole detectAvailableMechanismForStdout.
	
	arguments := self getSystemAttributes.
	arguments size > 0
		ifTrue: [scriptClassName := arguments at: 1.
			self executeRunFor: scriptClassName with: arguments.]! !

!ScriptStarter class methodsFor: 'system startup entry' stamp: 'sm 5/30/2011 22:30'!
startUp: resuming
	IsResuming := resuming.
	^ super startUp: resuming.! !

!ScriptStarter class methodsFor: 'transporter' stamp: 'StefanMarr 4/5/2012 21:48'!
transportersForFileOutMenu
	^ { (Smalltalk at: #Transporter ifAbsent: [^#()])
			forPackage: (PackageInfo named: 'Scripting') }! !

!ScriptStarter class methodsFor: 'system startup entry' stamp: ''!
uninstall
	"Will remove ScriptStarter from the StartUpList"
	Smalltalk removeFromStartUpList: ScriptStarter.
	
	self flag: 'TODO: restore original setting for readDocumentAtStartup, see >>install.'.
	Transcript show: 'You might need to restore the original setting for readDocumentAtStartup'.! !

Object subclass: #UpdateImage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Scripting'!

!UpdateImage class methodsFor: 'script entry' stamp: 'sm 5/30/2011 22:41'!
run: args
	| proc |

	"The loading needs to be done after everything in startup has been done.
	 Otherwise the snaphost routine gets confused.
	 Thus, we do that ourselfs."
	ScriptStarter quitVMAfterScriptExecution: false.
	
	"Use a delayed process to do the actual work"
	proc := [ (Delay forSeconds: 5) wait.
	  "Create an up-to-date image"
	  (FileStream readOnlyFileNamed: 'FileInAllRenaissanceCode.st') fileIn.
	Transcript show: 'Snapshot and Quit now...'. 
	 Smalltalk snapshot: true andQuit: true. ] newProcess.

	proc name: 'Update Image'.
	proc resume.
	! !
ScriptStarter initialize!
ScriptConsole initialize!
